@@ -74,7 +74,6 @@ __all__ = ['Statement', 'BeginStatement', 'EndStatement', 'Variable',
 import copy
 import logging
 
-from six import with_metaclass
 from fparser.common.readfortran import Line, Comment
 from fparser.common.utils import split_comma,       \
                                  specs_split_comma, \
@@ -82,7 +81,7 @@ from fparser.common.utils import split_comma,       \
 from fparser.common.utils import classes, AnalyzeError
 
 
-class AttributeHolder(object):
+class AttributeHolder():
     # copied from symbolic.base module
     """
     Defines a object with predefined attributes. Only those attributes
@@ -97,7 +96,6 @@ class AttributeHolder(object):
             self._attributes[k] = v
             if callable(v):
                 self._readonly.append(k)
-        return
 
     def __getattr__(self, name):
         if name not in self._attributes:
@@ -171,7 +169,7 @@ def get_base_classes(cls):
     return bases + cls.__bases__ + (cls, )
 
 
-class Variable(object, with_metaclass(classes)):
+class Variable(metaclass=classes):
     """
     Variable instance has attributes:
       name
@@ -200,7 +198,6 @@ class Variable(object, with_metaclass(classes)):
         # .is_array:
         #    rank
         #    shape
-        return
 
     def __repr__(self):
         line = []
@@ -220,7 +217,6 @@ class Variable(object, with_metaclass(classes)):
         if id(parent) not in list(map(id, self.parents)):
             self.parents.append(parent)
         self.parent = parent
-        return
 
     def set_type(self, typedecl):
         if self.typedecl is not None:
@@ -231,7 +227,6 @@ class Variable(object, with_metaclass(classes)):
                     % (self.name, self.typedecl.tostr(), typedecl.tostr()))
         assert typedecl is not None
         self.typedecl = typedecl
-        return
 
     def set_init(self, expr):
         if self.init is not None:
@@ -240,19 +235,16 @@ class Variable(object, with_metaclass(classes)):
                     'variable %r already has initialization %r, '
                     'resetting to %r' % (self.name, self.expr, expr))
         self.init = expr
-        return
 
     def set_dimension(self, dims):
-        import six
         dims = [tuple(dim.split(':')) for dim in dims]
-        dims = [tuple(map(six.text_type.strip, dim)) for dim in dims]
+        dims = [tuple(map(str.strip, dim)) for dim in dims]
         if self.dimension is not None:
             if not self.dimension == dims:
                 self.parent.warning(
                     'variable %r already has dimension %r, '
                     'resetting to %r' % (self.name, self.dimension, dims))
         self.dimension = dims
-        return
 
     def set_bounds(self, bounds):
         if self.bounds is not None:
@@ -261,7 +253,6 @@ class Variable(object, with_metaclass(classes)):
                     'variable %r already has bounds %r, '
                     'resetting to %r' % (self.name, self.bounds, bounds))
         self.bounds = bounds
-        return
 
     def set_length(self, length):
         if self.length is not None:
@@ -270,7 +261,6 @@ class Variable(object, with_metaclass(classes)):
                     'variable %r already has length %r, '
                     'resetting to %r' % (self.name, self.length, length))
         self.length = length
-        return
 
     known_intent_specs = ['IN', 'OUT', 'INOUT', 'CACHE', 'HIDE', 'COPY',
                           'OVERWRITE', 'CALLBACK', 'AUX', 'C', 'INPLACE',
@@ -285,7 +275,6 @@ class Variable(object, with_metaclass(classes)):
                     self.parent.warning('unknown intent-spec %r for %r'
                                         % (i, self.name))
                 self.intent.append(i)
-        return
 
     known_attributes = ['PUBLIC', 'PRIVATE', 'ALLOCATABLE', 'ASYNCHRONOUS',
                         'EXTERNAL', 'INTRINSIC', 'OPTIONAL', 'PARAMETER',
@@ -440,7 +429,6 @@ class Variable(object, with_metaclass(classes)):
                 if uattr not in self.known_attributes:
                     self.parent.warning('unknown attribute %r' % (attr))
                 attributes.append(uattr)
-        return
 
     def __str__(self):
         s = ''
@@ -542,7 +530,6 @@ class Variable(object, with_metaclass(classes)):
                             n = '(%s)-(%s)' % (spec[1], spec[0])
                         shape.append(str(n))
                 self.shape = shape
-        return
 
     def error(self, message):
         return self.parent.error(message)
@@ -554,11 +541,11 @@ class Variable(object, with_metaclass(classes)):
         return self.parent.info(message)
 
 
-class ProgramBlock(object, with_metaclass(classes)):
+class ProgramBlock(metaclass=classes):
     pass
 
 
-class Statement(object, with_metaclass(classes)):
+class Statement(metaclass=classes):
     """
     Statement instance has attributes:
       parent  - Parent BeginStatement or FortranParser instance
@@ -604,8 +591,6 @@ class Statement(object, with_metaclass(classes)):
 
         self.process_item()
 
-        return
-
     def __repr__(self):
         return self.torepr()
 
@@ -696,21 +681,17 @@ class Statement(object, with_metaclass(classes)):
     def error(self, message):
         message = self.format_message('ERROR', message)
         logging.getLogger(__name__).error(message)
-        return
 
     def warning(self, message):
         message = self.format_message('WARNING', message)
         logging.getLogger(__name__).warning(message)
-        return
 
     def info(self, message):
         message = self.format_message('INFO', message)
         logging.getLogger(__name__).info(message)
-        return
 
     def analyze(self):
         self.warning('nothing analyzed')
-        return
 
     def get_variable(self, name):
         """ Return Variable instance of variable name.
@@ -734,7 +715,7 @@ class Statement(object, with_metaclass(classes)):
         Returns dictonary containing statements that block provides or None
         when N/A.
         """
-        return
+        return None
 
 
 class BeginStatement(Statement):
@@ -774,7 +755,6 @@ class BeginStatement(Statement):
             self.name = '__'+self.blocktype.upper()+'__'
         self.construct_name = getattr(item, 'name', None)
         Statement.__init__(self, parent, item)
-        return
 
     def tostr(self):
         return self.blocktype.upper() + ' ' + self.name
@@ -809,7 +789,6 @@ class BeginStatement(Statement):
         if item is None:
             return
         self.fill()
-        return
 
     def fill(self, end_flag=False):
         """
@@ -836,7 +815,6 @@ class BeginStatement(Statement):
 
         if not end_flag:
             self.warning('failed to find the end of block')
-        return
 
     def process_subitem(self, item):
         """
@@ -925,10 +903,9 @@ class BeginStatement(Statement):
                     self.reader.set_mode(False, False)
                 return r
 
-        self.handle_unknown_item(item)
-        return
+        self.handle_unknown_item_and_raise(item)
 
-    def handle_unknown_item(self, item):
+    def handle_unknown_item_and_raise(self, item):
         '''Called when process_subitem does not find a start or end of block.
         It adds the item (which is an instance of Line) to the content, but
         then raises an AnalyzeError. An instance of Line in content typically
@@ -947,7 +924,6 @@ class BeginStatement(Statement):
     def analyze(self):
         for stmt in self.content:
             stmt.analyze()
-        return
 
 
 class EndStatement(Statement):
@@ -996,7 +972,7 @@ class EndStatement(Statement):
         self.name = name
 
     def analyze(self):
-        return
+        pass
 
     def get_indent_tab(self, deindent=False, isfix=None):
         return Statement.get_indent_tab(self, deindent=True, isfix=isfix)
