@@ -1,7 +1,7 @@
 #!/usr/bin/env python
 # -*- coding: utf-8 -*-
-# Modified work Copyright (c) 2017-2020 Science and Technology
-# Facilities Council
+# Modified work Copyright (c) 2017-2022 Science and Technology
+# Facilities Council.
 # Original work Copyright (c) 1999-2008 Pearu Peterson
 
 # All rights reserved.
@@ -67,7 +67,7 @@
 #
 # Author: Pearu Peterson <pearu@cens.ioc.ee>
 # Created: May 2006
-# Modified by R. W. Ford, STFC Daresbury Lab
+# Modified by R. W. Ford and A. R. Porter, STFC Daresbury Lab
 # Modified by P. Elson, Met Office
 
 """Provides Fortran reader classes.
@@ -139,15 +139,12 @@ To read a Fortran code from a string, use `FortranStringReader` class::
 
 """
 
-from __future__ import print_function
-
 import logging
-import io
 import os
 import re
 import sys
 import traceback
-import six
+from io import StringIO
 import fparser.common.sourceinfo
 from fparser.common.splitline import String, string_replace_map, splitquote
 
@@ -173,19 +170,26 @@ def _is_fix_cont(line):
     return line and len(line) > 5 and line[5] != ' ' and line[:5] == 5 * ' '
 
 
-def _is_fix_comment(line, isstrict):
-    """
+def _is_fix_comment(line, isstrict, f2py_enabled):
+    '''
     Check whether line is a comment line in fixed format Fortran source.
 
-    :param str line: Line of code to check
-    :param bool isstrict: Whether we are strictly enforcing fixed/free fmt
-
     References
     ----------
     :f2008:`3.3.3`
-    """
+
+    :param str line: line of code to check.
+    :param bool isstrict: whether we are strictly enforcing fixed/free fmt.
+    :param bool f2py_enabled: whether support for f2py directives is enabled.
+
+    :returns: whether or not the supplied line is a fixed-format comment.
+    :rtype: bool
+
+    '''
     if line:
         if line[0] in '*cC!':
+            if f2py_enabled and line[1:5].lower() == "f2py":
+                return False
             return True
         if not isstrict:
             i = line.find('!')
@@ -261,7 +265,7 @@ class FortranReaderError(Exception):
     pass
 
 
-class Line(object):
+class Line():
     """ Holds a Fortran source line.
 
     Attributes
@@ -279,10 +283,6 @@ class Line(object):
     is_f2py_directive : bool
       the line contains f2py directive
     """
-
-    f2py_strmap_findall = re.compile(r'(_F2PY_STRING_CONSTANT_\d+_'
-                                     + r'|F2PY_EXPR_TUPLE_\d+)').findall
-
     def __init__(self, line, linenospan, label, name, reader):
         self.line = line.strip()
         if not self.line:
@@ -311,12 +311,7 @@ class Line(object):
         '''
         if not hasattr(self, 'strlinemap') or not self.strlinemap:
             return line
-        findall = self.f2py_strmap_findall
-        str_map = self.strlinemap
-        keys = findall(line)
-        for k in keys:
-            line = line.replace(k, str_map[k])
-        return line
+        return self.strlinemap(line)
 
     def copy(self, line=None, apply_map=False):
         '''
@@ -339,7 +334,6 @@ class Line(object):
         '''
         self.line = self.apply_map(line)
         self.strline = None
-        return
 
     def __repr__(self):
         return self.__class__.__name__+'(%r,%s,%r,%r,<reader>)' \
@@ -427,7 +421,7 @@ class SyntaxErrorLine(Line, FortranReaderError):
         FortranReaderError.__init__(self, message)
 
 
-class Comment(object):
+class Comment():
     '''Holds a Fortran comment.
 
     :param str comment: String containing the text of a single or \
@@ -467,7 +461,7 @@ class Comment(object):
         return ignore_comments
 
 
-class MultiLine(object):
+class MultiLine():
     """ Holds PYF file multiline.
 
     PYF file multiline is represented as follows::
@@ -535,7 +529,7 @@ class CppDirective(Line):
 ##############################################################################
 
 
-class FortranReaderBase(object):
+class FortranReaderBase():
     """
     Base class for reading Fortran sources.
 
@@ -544,14 +538,14 @@ class FortranReaderBase(object):
     code, free format Fortran code, or PYF signatures (with extended
     free format Fortran syntax).
 
-    :param source: A file-like object with .next() method used to
+    :param source: a file-like object with .next() method used to \
                    retrive a line.
-    :type source: either :py:class:`six.StringIO` or a file handle
-    :param mode: A FortranFormat object as returned by
+    :type source: :py:class:`StringIO` or a file handle
+    :param mode: a FortranFormat object as returned by \
                  `sourceinfo.get_source_info()`
     :type mode: :py:class:`fparser.common.sourceinfo.Format`
-    :param bool isstrict: Whether we are strictly enforcing fixed format
-    :param bool ignore_comments: Whether or not to discard comments
+    :param bool isstrict: whether we are strictly enforcing fixed format.
+    :param bool ignore_comments: whether or not to discard comments.
 
     The Fortran source is iterated by `get_single_line`,
     `get_next_line`, `put_single_line` methods.
@@ -582,7 +576,6 @@ class FortranReaderBase(object):
         self.exit_on_error = True
         self.restore_cache = []
 
-        return
 
     ##########################################################################
 
@@ -654,7 +647,6 @@ class FortranReaderBase(object):
         """
         self.filo_line.append(line)
         self.linecount -= 1
-        return
 
     def get_single_line(self, ignore_empty=False, ignore_comments=None):
         """ Return line from FILO line buffer or from source.
@@ -670,8 +662,8 @@ class FortranReaderBase(object):
         In both situations ``linecount`` will be incremented, that is,
         the line will be consumed.
 
-        :param bool ignore_empty: If True then ignore empty lines.
-        :param bool ignore_comments: If True then ignore comments (overrides
+        :param bool ignore_empty: if True then ignore empty lines.
+        :param bool ignore_comments: if True then ignore comments (overrides \
                                      self._ignore_comments)
 
         See also
@@ -697,32 +689,17 @@ class FortranReaderBase(object):
             return None
         self.linecount += 1
 
-        if six.PY2 and not isinstance(line, six.text_type):
-            # Ensure we always have a unicode object in Python 2.
-            line = unicode(line, 'UTF-8')
-
         # expand tabs, replace special symbols, get rid of nl characters
         line = line.expandtabs().replace(u'\xa0', u' ').rstrip()
 
-        if six.PY2:
-            # Cast the unicode to str if we can do so safely. This
-            # maximises compatibility with the existing Python 2 tests
-            # and avoids the need to proliferate the use of unicode
-            # literals (e.g. u"") in the parse tree repr.
-            try:
-                line = line.encode('ascii', errors='strict')
-            except UnicodeEncodeError:
-                # Can't cast to str as there are non-ascii characters
-                # in the line.
-                pass
-
         self.source_lines.append(line)
 
-        if ignore_comments and (self.format.is_fixed or self.format.is_f77):
+        if ignore_comments and (self._format.is_fixed or self._format.is_f77):
             # Check for a fixed-format comment. If the current line *is*
             # a comment and we are ignoring them, then recursively call this
             # routine again to get the next source line.
-            if _is_fix_comment(line, isstrict=self.format.is_strict):
+            if _is_fix_comment(line, isstrict=self._format.is_strict,
+                               f2py_enabled=self._format.f2py_enabled):
                 return self.get_single_line(ignore_empty, ignore_comments)
 
         if ignore_empty and not line:
@@ -766,7 +743,6 @@ class FortranReaderBase(object):
         """ Insert item to FIFO item buffer.
         """
         self.fifo_item.insert(0, item)
-        return
 
     # Iterator methods:
 
@@ -859,7 +835,7 @@ class FortranReaderBase(object):
             logging.getLogger(__name__).critical(message)
             message = 'Traceback\n' + ''.join(traceback.format_stack())
             logging.getLogger(__name__).debug(message)
-            logging.getLogger(__name__).debug(six.text_type(err))
+            logging.getLogger(__name__).debug(str(err))
             logging.getLogger(__name__).critical('STOPPED READING')
             raise StopIteration
 
@@ -870,12 +846,18 @@ class FortranReaderBase(object):
 
         Resolves ``;`` statement terminations.
 
-        :param bool ignore_comments: Whether or not to ignore comments
-                                     (overrides self._ignore_comments)
-
         See also
         --------
         next, get_source_item
+
+        :param bool ignore_comments: Whether or not to ignore comments \
+                                     (overrides self._ignore_comments)
+
+        :returns: the next line of Fortran.
+        :rtype: :py:class:`fparser.common.readfortran.Line`
+
+        :raises StopIteration: if no new items are found.
+
         """
         if ignore_comments is None:
             ignore_comments = self._ignore_comments
@@ -887,9 +869,9 @@ class FortranReaderBase(object):
             except IndexError:
                 # construct a new item from source
                 item = self.get_source_item()
-                if item is None:
-                    raise StopIteration
-            if not (item.isempty(ignore_comments)):
+            if item is None:
+                raise StopIteration
+            if not item.isempty(ignore_comments):
                 break
             # else ignore empty lines and comments by getting next line
 
@@ -1046,7 +1028,6 @@ class FortranReaderBase(object):
                                     message,
                                     item.span[0], item.span[1])
         logging.getLogger(__name__).info(m)
-        return
 
     def error(self, message, item=None):
         '''
@@ -1060,7 +1041,6 @@ class FortranReaderBase(object):
         logging.getLogger(__name__).error(m)
         if self.exit_on_error:
             sys.exit(1)
-        return
 
     def warning(self, message, item=None):
         '''
@@ -1075,7 +1055,6 @@ class FortranReaderBase(object):
                                             item.span[0],
                                             item.span[1])
         logging.getLogger(__name__).warning(m)
-        return
 
     # Auxiliary methods for processing raw source lines:
 
@@ -1104,22 +1083,24 @@ class FortranReaderBase(object):
         return (line, line.lstrip().startswith('#'))
 
     def handle_cf2py_start(self, line):
-        """ Apply f2py directives to line.
+        '''
+        Process any f2py directives contained in the supplied line. If
+        support for such directives has been disabled then the line is
+        returned unchanged.
 
         F2py directives are specified in the beginning of the line.
 
         f2py directives can be used only in Fortran codes.  They are
         ignored when used inside PYF files.
 
-        Parameters
-        ----------
-        line : str
+        :param str line: the line to check for f2py directives.
 
-        Returns
-        -------
-        line : str
-        """
-        if not line or self._format.is_pyf:
+        :returns: the line with any f2py directives applied (if they are \
+                  enabled in the reader).
+        :rtype: str
+
+        '''
+        if not line or self._format.is_pyf or not self._format.f2py_enabled:
             return line
         if self._format.is_fixed:
             if line[0] in '*cC!#':
@@ -1184,7 +1165,7 @@ class FortranReaderBase(object):
             commentline = ''.join(items[k:])
             break
         if commentline is not None:
-            if commentline.startswith('!f2py'):
+            if self._format.f2py_enabled and commentline.startswith('!f2py'):
                 # go to next iteration:
                 newline = ''.join(noncomment_items) + commentline[5:]
                 self.f2py_comment_lines.append(lineno)
@@ -1272,7 +1253,8 @@ class FortranReaderBase(object):
     # the following contexts: f77, fixed, free, pyf.
 
     def get_source_item(self):
-        """ Return next source item.
+        '''
+        Return the next source item.
 
         A source item is ..
         - a fortran line
@@ -1280,11 +1262,20 @@ class FortranReaderBase(object):
         - a multiline - lines inside triple-quotes, only when in ispyf mode
         - a comment line
         - a preprocessor directive line
-        """
+
+        :returns: the next source item.
+        :rtype: :py:class:`fparser.common.readfortran.Line` or \
+            :py:class:`fparser.common.readfortran.MultiLine` or \
+            :py:class:`fparser.common.readfortran.Comment` or \
+            :py:class:`fparser.common.readfortran.CppDirective` or \
+            :py:class:`fparser.common.readfortran.SyntaxErrorLine` or \
+            :py:class:`fparser.common.readfortran.SyntaxErrorMultiLine`
+
+        '''
         get_single_line = self.get_single_line
         line = get_single_line()
         if line is None:
-            return
+            return None
         startlineno = self.linecount
         line, is_cpp_directive = self.handle_cpp_directive(line)
         if is_cpp_directive:
@@ -1300,7 +1291,8 @@ class FortranReaderBase(object):
                                            endlineno)
 
         line = self.handle_cf2py_start(line)
-        is_f2py_directive = startlineno in self.f2py_comment_lines
+        is_f2py_directive = (self._format.f2py_enabled and
+                             startlineno in self.f2py_comment_lines)
         isstrict = self._format.is_strict
         have_comment = False
         label = None
@@ -1309,11 +1301,11 @@ class FortranReaderBase(object):
         if self._format.is_pyf:
             # handle multilines
             for mlstr in ['"""', "'''"]:
-                r = self.handle_multilines(line, startlineno, mlstr)
-                if r:
-                    return r
+                multiline = self.handle_multilines(line, startlineno, mlstr)
+                if multiline:
+                    return multiline
         if self._format.is_fixed:
-            if _is_fix_comment(line, isstrict):
+            if _is_fix_comment(line, isstrict, self._format.f2py_enabled):
                 # comment line:
                 return self.comment_item(line, startlineno, startlineno)
 
@@ -1409,14 +1401,15 @@ class FortranReaderBase(object):
             lines = [newline]
             next_line = self.get_next_line()
 
-            while _is_fix_cont(next_line) or _is_fix_comment(next_line,
-                                                             isstrict):
+            while (_is_fix_cont(next_line) or
+                   _is_fix_comment(next_line, isstrict,
+                                   self._format.f2py_enabled)):
                 # handle fix format line continuations for F90 or
                 # newer code.  Mixing fix format and free format line
                 # continuations is not allowed nor detected, just
                 # eject warnings.
                 line2 = get_single_line()  # consume next_line as line2
-                if _is_fix_comment(line2, isstrict):
+                if _is_fix_comment(line2, isstrict, self._format.f2py_enabled):
                     # handle fix format comments inside line continuations
                     # after the line construction
                     citem = self.comment_item(line2,
@@ -1551,9 +1544,15 @@ class FortranReaderBase(object):
             self.warning(message)
         if name is not None:
             self.error('No construct following construct-name.')
-        if have_comment:
-            return next(self)
-        return self.comment_item('', startlineno, endlineno)
+
+        # If this point is reached, the line is a comment or is
+        # blank. If it is a comment, it has been pushed onto the
+        # fifo_item list.
+        try:
+            return self.fifo_item.pop(0)
+        except IndexError:
+            # A blank line is represented as an empty comment
+            return Comment('', (startlineno, endlineno), self)
 
 
 class FortranFileReader(FortranReaderBase):
@@ -1579,18 +1578,14 @@ class FortranFileReader(FortranReaderBase):
         # contents of the file. Obviously if the file changes content but not
         # filename, problems will ensue.
         #
-        self._remove_on_destruction = False
         self._close_on_destruction = False
-        if isinstance(file_candidate, six.string_types):
+        if isinstance(file_candidate, str):
             self.id = file_candidate
-            from fparser.common.utils import make_clean_tmpfile
-            # Handle potential invalid characters in the input. Done
-            # by creating a new file (tmpfile) with any errors removed
-            # (or raising an exception - see make_clean_tmpfile).
-            tmpfile = make_clean_tmpfile(file_candidate)
-            self.file = io.open(tmpfile, 'r', encoding='UTF-8')
+            # The 'fparser-logging' handler for errors ensures that any invalid
+            # characters in the input are skipped but logged.
+            self.file = open(file_candidate, 'r', encoding='UTF-8',
+                             errors='fparser-logging')
             self._close_on_destruction = True
-            self._remove_on_destruction = True
         elif hasattr(file_candidate,
                      'read') and hasattr(file_candidate,
                                          'name'):  # Is likely a file
@@ -1610,13 +1605,10 @@ class FortranFileReader(FortranReaderBase):
             self.include_dirs = include_dirs[:]
         if source_only is not None:
             self.source_only = source_only[:]
-        return
 
     def __del__(self):
         if self._close_on_destruction:
             self.file.close()
-            if self._remove_on_destruction:
-                os.remove(self.file.name)
 
     def close_source(self):
         self.file.close()
@@ -1656,7 +1648,7 @@ class FortranStringReader(FortranReaderBase):
         # anyway.
         #
         self.id = 'string-' + str(hash(string))
-        source = six.StringIO(string)
+        source = StringIO(string)
         mode = fparser.common.sourceinfo.get_source_info_str(string)
         FortranReaderBase.__init__(self, source, mode,
                                    ignore_comments)
@@ -1664,4 +1656,3 @@ class FortranStringReader(FortranReaderBase):
             self.include_dirs = include_dirs[:]
         if source_only is not None:
             self.source_only = source_only[:]
-        return
