@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 ##############################################################################
-# Copyright (c) 2017-2020 Science and Technology Facilities Council
+# Copyright (c) 2017-2022 Science and Technology Facilities Council
 #
 # All rights reserved.
 #
@@ -40,21 +40,28 @@
 '''
 Test battery associated with fparser.common.readfortran package.
 '''
-from __future__ import print_function
 
 import io
 import os.path
 import tempfile
 import re
-import six
 import pytest
 
 from fparser.common.readfortran import FortranFileReader, \
     FortranStringReader, FortranReaderBase, Line, extract_label, \
-    extract_construct_name, CppDirective
+    extract_construct_name, CppDirective, Comment
 from fparser.common.sourceinfo import FortranFormat
 
 
+@pytest.fixture(scope="module",
+                name="f2py_enabled",
+                params=[True, False])
+def f2py_enabled_fixture(request):
+    ''' Fixture that returns whether or not to enable reader support for
+    f2py directives. '''
+    return request.param
+
+
 def test_empty_line_err():
     ''' Check that we raise the expected error if we try and create
     an empty Line '''
@@ -64,28 +71,49 @@ def test_empty_line_err():
     assert "Got empty line: \'   \'" in str(err.value)
 
 
+def test_line_map():
+    ''' Check the application and reversal of tokenisation of a Line for
+    strings, expressions in parentheses and literal constants with an
+    exponent.
+
+    '''
+    my_code = ("program test\n"
+               " real :: var\n"
+               " var = 1.0e-3\n"
+               " var = var * (var + 1.0d-4)\n"
+               " write(*,*) 'var = ', var\n"
+               "end program test\n")
+    reader = FortranStringReader(my_code, ignore_comments=True)
+    for _ in range(3):
+        line = reader.next()
+    assert line.get_line() == "var = F2PY_REAL_CONSTANT_1_"
+    assert line.get_line(apply_map=True) == "var = 1.0e-3"
+    line = reader.next()
+    assert line.get_line() == "var = var * (F2PY_EXPR_TUPLE_1)"
+    assert line.get_line(apply_map=True) == "var = var * (var + 1.0d-4)"
+    line = reader.next()
+    assert line.get_line() == ("write(F2PY_EXPR_TUPLE_1) "
+                               "'_F2PY_STRING_CONSTANT_1_', var")
+    assert line.get_line(apply_map=True) == "write(*,*) 'var = ', var"
+
+
 def test_111fortranreaderbase(log, monkeypatch):
     '''
     Tests the FortranReaderBase class.
 
     Currently only tests logging functionality.
     '''
-    class FailFile(object):
+    class FailFile():
         '''
         A "file-like" object which returns a line of Fortran source followed
         by raising a StopIteration exception.
         '''
         _stuff = ['x=1']
 
-        def next(self):
-            '''
-            Used by Python 2.7.
-            '''
-            return self.__next__()
-
         def __next__(self):
             '''
-            Used by Python 3.
+            :returns: the next line of source.
+            :rtype: str
             '''
             return self._stuff.pop()
 
@@ -346,7 +374,7 @@ def check_include_works(fortran_filename, fortran_code, include_info,
     file specified by 'fortran_filename'.
     :param include_info: a list of 2-tuples each with an include \
     filename as a string followed by include code as a string.
-    :type include_info: list of (str, str)
+    :type include_info: List[str]
     :param str expected: the expected output after parsing the code.
     :param str tmpdir: the temporary directory in which to create and \
     process the Fortran files.
@@ -745,7 +773,32 @@ def test_file_reader():
         raise
 
 
-##############################################################################
+def test_none_in_fifo(log):
+    ''' Check that a None entry in the reader FIFO buffer is handled
+    correctly. '''
+    log.reset()
+    handle, filename = tempfile.mkstemp(suffix='.f90', text=True)
+    os.close(handle)
+
+    with io.open(filename, mode='w', encoding='UTF-8') as source_file:
+        source_file.write(FULL_FREE_SOURCE)
+
+    with io.open(filename, mode='r', encoding='UTF-8') as source_file:
+        unit_under_test = FortranFileReader(source_file)
+        while True:
+            try:
+                _ = unit_under_test.next(ignore_comments=False)
+            except StopIteration:
+                break
+        # Erroneously push a None to the FIFO buffer
+        unit_under_test.put_item(None)
+        # Attempt to read the next item
+        with pytest.raises(StopIteration):
+            _ = unit_under_test.next(ignore_comments=False)
+        # Check that nothing has been logged
+        for log_level in ["debug", "info", "warning", "error", "critical"]:
+            assert log.messages[log_level] == []
+
 
 def test_bad_file_reader():
     '''
@@ -784,7 +837,6 @@ c12346 comment
      'bar
 a    'g
       abc=2
-cf2py call me ! hey
       call you ! hi
       end
      '"""
@@ -794,8 +846,8 @@ cf2py call me ! hey
                 "line #4'call foobar'",
                 'Comment("a    \'g",(6, 6))',
                 "line #7'abc=2'",
-                "line #9'call you ! hi'",
-                "line #10'end'"]
+                "line #8'call you ! hi'",
+                "line #9'end'"]
 
     # Reading from buffer
     reader = FortranStringReader(
@@ -886,8 +938,7 @@ end python module foo
         string_pyf, ignore_comments=False)
     assert reader.format.mode == 'pyf', repr(reader.format.mode)
     for item in reader:
-        # Remove 'u's to allow for py2/3 unicode differences
-        assert re.sub("u", "", str(item)) == re.sub("u", "", expected.pop(0))
+        assert str(item) == expected.pop(0)
 
 
 def test_fix90():
@@ -902,7 +953,6 @@ cComment
 !
      !4!line cont. with comment symbol
      &5
-      a = 3!f2py.14 ! pi!
 !   KDMO
       write (obj%print_lun, *) ' KDMO : '
       write (obj%print_lun, *) '  COORD = ',coord, '  BIN_WID = ',             &
@@ -921,24 +971,90 @@ cComment
                 "line #5'b = 345'",
                 "Comment('!',(6, 6))",
                 "Comment('!line cont. with comment symbol',(7, 7))",
-                "line #9'a = 3.14'",
-                "Comment('! pi!',(9, 9))",
-                "Comment('!   KDMO',(10, 10))",
-                'line #11"write (obj%print_lun, *) \' KDMO : \'"',
-                'line #12"write (obj%print_lun, *) \'  COORD = \',coord,'
+                "Comment('!   KDMO',(9, 9))",
+                'line #10"write (obj%print_lun, *) \' KDMO : \'"',
+                'line #11"write (obj%print_lun, *) \'  COORD = \',coord,'
                 + ' \'  BIN_WID = \',             &"',
-                'line #13"obj%bin_wid,\'  VEL_DMO = \', obj%vel_dmo"',
-                "line #14'end subroutine foo'",
-                "line #15'subroutine foo'",
-                "Comment('',(16, 16))",
-                "line #18'end'"]
+                'line #12"obj%bin_wid,\'  VEL_DMO = \', obj%vel_dmo"',
+                "line #13'end subroutine foo'",
+                "line #14'subroutine foo'",
+                "Comment('',(15, 15))",
+                "line #17'end'"]
     reader = FortranStringReader(
         string_fix90, ignore_comments=False)
     assert reader.format.mode == 'fix', repr(reader.format.mode)
     for item in reader:
-        # Remove 'u's to allow for py2/3 unicode differences
-        assert re.sub("u", "", six.text_type(item)) == \
-            re.sub("u", "", expected.pop(0))
+        assert str(item) == expected.pop(0)
+
+
+def test_f2py_directive_fixf90(f2py_enabled):
+    ''' Test the handling of the f2py directive in fixed-format f90. '''
+    string_fix90 = """c -*- fix -*-
+      subroutine foo
+      a = 3!f2py.14 ! pi!
+!f2py a = 0.0
+      end subroutine foo"""
+    reader = FortranStringReader(string_fix90, ignore_comments=False)
+    reader.set_format(FortranFormat(False, False, f2py_enabled))
+    expected = ["Comment('c -*- fix -*-',(1, 1))",
+                "line #2'subroutine foo'"]
+    if f2py_enabled:
+        expected.extend(["line #3'a = 3.14'",
+                         "Comment('! pi!',(3, 3))",
+                         "line #4'a = 0.0'"])
+    else:
+        expected.extend(["line #3'a = 3'",
+                         "Comment('!f2py.14 ! pi!',(3, 3))",
+                         "Comment('!f2py a = 0.0',(4, 4))"])
+    expected.append("line #5'end subroutine foo'")
+    for item in reader:
+        assert str(item) == expected.pop(0)
+
+
+def test_f2py_freef90(f2py_enabled):
+    ''' Test the handling of f2py directives in free-format f90, in both
+    in-line and full-line comments. '''
+    lines = ["subroutine foo",
+             "   a = 3!f2py.14 ! pi!",
+             "!f2py a = 0.0",
+             "end subroutine foo"]
+    reader = FortranStringReader("\n".join(lines), ignore_comments=False)
+    reader.set_format(FortranFormat(True, False, f2py_enabled))
+    expected = ["line #1'subroutine foo'"]
+    if f2py_enabled:
+        expected.extend(["line #2'a = 3.14'",
+                         "Comment('! pi!',(2, 2))",
+                         "line #3'a = 0.0'"])
+    else:
+        expected.extend(["line #2'a = 3'",
+                         "Comment('!f2py.14 ! pi!',(2, 2))",
+                         "Comment('!f2py a = 0.0',(3, 3))"])
+    expected.append("line #4'end subroutine foo'")
+    for item in reader:
+        assert str(item) == expected.pop(0)
+
+
+@pytest.mark.xfail(reason="Issue #270: f2py directives not working in F77 "
+                   "code.")
+def test_f2py_directive_f77(f2py_enabled):
+    ''' Test the handling of the f2py directive in fixed-format f77. '''
+    string_f77 = """c -*- f77 -*-
+      subroutine foo
+cf2py call me ! hey
+      end"""
+    expected = ["Comment('c -*- f77 -*-',(1, 1))",
+                "line #2'subroutine foo'"]
+    if f2py_enabled:
+        expected.append("line #3'call me ! hey'")
+    else:
+        expected.append("Comment('cf2py call me ! hey',(3, 3))")
+    expected.append("line #4'end'")
+
+    # Reading from buffer
+    reader = FortranStringReader(
+        string_f77, ignore_comments=False)
+    for item in reader:
+        assert str(item) == expected.pop(0)
 
 
 def test_utf_char_in_code(log):
@@ -1017,7 +1133,7 @@ def test_reader_cpp_directives():
         'end program test']
     ref_text = '\n'.join(input_text[:4] + input_text[5:]).strip()
     reader = FortranStringReader('\n'.join(input_text))
-    lines = [item for item in reader]
+    lines = list(reader)
 
     pp_lines = [1, 3, 5, 6, 8]
     for i, line in enumerate(lines):
@@ -1036,8 +1152,192 @@ def test_multiline_cpp_directives():
         'integer a', 'end program test']
     ref_text = 'program test\n#define ABC   123\ninteger a\nend program test'
     reader = FortranStringReader('\n'.join(input_text))
-    lines = [item for item in reader]
+    lines = list(reader)
     assert len(lines) == 4
     assert isinstance(lines[1], CppDirective)
     assert lines[1].span == (2, 3)
     assert '\n'.join(item.line for item in lines) == ref_text
+
+
+# Tests for the get_source_item method in class FortranReaderBase :
+# Comments
+
+
+def test_single_comment():
+    '''Test that a free format single line comment is output as expected'''
+    input_text = "! a comment\n"
+
+    reader = FortranStringReader(input_text, ignore_comments=False)
+    lines = list(reader)
+    assert len(lines) == 1
+    assert isinstance(lines[0], Comment)
+    assert lines[0].span == (1, 1)
+    assert lines[0].line + "\n" == input_text
+
+    reader = FortranStringReader(input_text, ignore_comments=True)
+    lines = list(reader)
+    assert len(lines) == 0
+
+
+def test_many_comments():
+    '''Test that a large consecutive number of single line comments can be
+    successfully processed. Earlier versions of the reader used
+    recursion for multiple consecutive free format single line
+    comments which resulted in a recursion error in this case.
+
+    '''
+    number_of_comments = 1000
+    input_text = "program hello\n"
+    for index in range(number_of_comments):
+        input_text += "! comment{0}\n".format(index+1)
+    input_text += "end program hello\n"
+
+    reader = FortranStringReader(input_text, ignore_comments=False)
+    lines = list(reader)
+    assert len(lines) == number_of_comments + 2
+    for index in range(1, number_of_comments):
+        assert isinstance(lines[index], Comment)
+        assert lines[index].span == (index+1, index+1)
+        assert lines[index].line + "\n" == "! comment{0}\n".format(index)
+
+    reader = FortranStringReader(input_text, ignore_comments=True)
+    lines = list(reader)
+    assert len(lines) == 2
+
+
+def test_comments_within_continuation():
+    '''Test that comments and multi-line statements are processed
+    correctly.
+
+    '''
+    input_text = (
+        "  ! Comment1\n"
+        "  real :: a &\n"
+        "  ! Comment2\n"
+        "          ,b\n"
+        "  ! Comment3\n")
+
+    reader = FortranStringReader(input_text, ignore_comments=False)
+    lines = list(reader)
+    assert len(lines) == 4
+
+    assert isinstance(lines[0], Comment)
+    assert lines[0].span == (1, 1)
+    assert lines[0].line == "! Comment1"
+
+    assert lines[1].span == (2, 4)
+    assert lines[1].line == "real :: a           ,b"
+
+    assert isinstance(lines[2], Comment)
+    assert lines[2].span == (3, 3)
+    assert lines[2].line == "! Comment2"
+
+    assert isinstance(lines[3], Comment)
+    assert lines[3].span == (5, 5)
+    assert lines[3].line == "! Comment3"
+
+    reader = FortranStringReader(input_text, ignore_comments=True)
+    lines = list(reader)
+    assert len(lines) == 1
+    assert lines[0].span == (2, 4)
+    assert lines[0].line == "real :: a           ,b"
+
+
+# Tests for the get_source_item method in class FortranReaderBase :
+# Blank lines
+
+
+@pytest.mark.parametrize("input_text", ["\n", "    \n"])
+def test_single_blank_line(input_text):
+    '''Test that a single blank line with or without white space is output
+    as an empty Comment object.
+
+    '''
+    reader = FortranStringReader(input_text, ignore_comments=False)
+    lines = list(reader)
+    assert len(lines) == 1
+    assert isinstance(lines[0], Comment)
+    assert lines[0].span == (1, 1)
+    assert lines[0].line == ""
+
+    reader = FortranStringReader(input_text, ignore_comments=True)
+    lines = list(reader)
+    assert len(lines) == 0
+
+
+def test_multiple_blank_lines():
+    '''Test that multiple blank lines with or without white space are
+    output as an empty Comment objects.
+
+    '''
+    input_text = (
+        "   \n\n"
+        "program test\n"
+        "  \n\n"
+        "end program test\n"
+        "  \n\n")
+    reader = FortranStringReader(input_text, ignore_comments=False)
+    lines = list(reader)
+    assert len(lines) == 8
+    for index in [0, 1, 3, 4, 6, 7]:
+        assert isinstance(lines[index], Comment)
+        assert lines[index].span == (index+1, index+1)
+        assert lines[index].line == ""
+    assert isinstance(lines[2], Line)
+    assert lines[2].span == (3, 3)
+    assert lines[2].line == "program test"
+    assert isinstance(lines[5], Line)
+    assert lines[5].span == (6, 6)
+    assert lines[5].line == "end program test"
+
+    reader = FortranStringReader(input_text, ignore_comments=True)
+    lines = list(reader)
+    assert len(lines) == 2
+    assert isinstance(lines[0], Line)
+    assert lines[0].span == (3, 3)
+    assert lines[0].line == "program test"
+    assert isinstance(lines[1], Line)
+    assert lines[1].span == (6, 6)
+    assert lines[1].line == "end program test"
+
+
+def test_blank_lines_within_continuation():
+    '''Test that blank lines and multi-line statements are processed
+    correctly. Note, empty lines within a multi-line statement are
+    removed.
+
+    '''
+    input_text = (
+        "  \n"
+        "  real :: a &\n"
+        "  \n\n"
+        "          ,b\n"
+        "  \n"
+        "  real :: c\n")
+
+    reader = FortranStringReader(input_text, ignore_comments=False)
+    lines = list(reader)
+    assert len(lines) == 4
+
+    assert isinstance(lines[0], Comment)
+    assert lines[0].span == (1, 1)
+    assert lines[0].line == ""
+    assert isinstance(lines[1], Line)
+    assert lines[1].span == (2, 5)
+    assert lines[1].line == "real :: a           ,b"
+    assert isinstance(lines[2], Comment)
+    assert lines[2].span == (6, 6)
+    assert lines[2].line == ""
+    assert isinstance(lines[3], Line)
+    assert lines[3].span == (7, 7)
+    assert lines[3].line == "real :: c"
+
+    reader = FortranStringReader(input_text, ignore_comments=True)
+    lines = list(reader)
+    assert len(lines) == 2
+    assert isinstance(lines[0], Line)
+    assert lines[0].span == (2, 5)
+    assert lines[0].line == "real :: a           ,b"
+    assert isinstance(lines[1], Line)
+    assert lines[1].span == (7, 7)
+    assert lines[1].line == "real :: c"
