@@ -68,8 +68,6 @@ Fortran single line statements.
 
 """
 
-from __future__ import print_function
-
 import re
 import sys
 
@@ -687,10 +685,9 @@ class Allocate(Statement):
         :return: Fortran code
         :rtype: str
         '''
-        import six
         type_spec = ''
         if self.spec:
-            if isinstance(self.spec, six.text_type):
+            if isinstance(self.spec, str):
                 type_spec = self.spec
             else:
                 type_spec = self.spec.tostr()
