@@ -80,7 +80,7 @@ import re
 dollar_ok = True
 
 
-class Pattern(object):
+class Pattern():
     """
     p1 | p2    -> <p1> | <p2>
     p1 + p2    -> <p1> <p2>
@@ -120,7 +120,6 @@ class Pattern(object):
         self.optional = optional
         self._flags = flags
         self.value = value
-        return
 
     def flags(self, *flags):
         f = self._flags
@@ -474,6 +473,14 @@ attr_spec = Pattern(
     flags=re.I)
 abs_attr_spec = abs(attr_spec)
 
+attr_spec_f08 = Pattern(
+    '<attr-spec>',
+    r'({})'.format('|'.join(  # extend attr_spec with attribute CONTIGUOUS
+        sorted(attr_spec.pattern.strip('()').split('|') + ['CONTIGUOUS'])
+    )),
+    flags=re.I)
+abs_attr_spec_f08 = abs(attr_spec_f08)
+
 dimension = Pattern('<dimension>', r'DIMENSION', flags=re.I)
 abs_dimension = abs(dimension)
 
