@@ -1,4 +1,4 @@
-# Copyright (c) 2019-2020 Science and Technology Facilities Council
+# Copyright (c) 2019-2021 Science and Technology Facilities Council
 
 # All rights reserved.
 
@@ -47,7 +47,7 @@ import sys
 import pytest
 
 import fparser.two.Fortran2003 as f2003
-import fparser.two.utils
+from fparser.two.utils import NoMatchError
 
 
 def assert_subclass_parse(source, base_type, actual_type=None,
@@ -109,7 +109,8 @@ def possible_subclasses(node_type, _seen=None):
     return seen
 
 
-def test_intrinsic_function(f2003_create):
+@pytest.mark.usefixtures("f2003_create", "fake_symbol_table")
+def test_intrinsic_function():
     '''Test that an intrinsic function is matched by Primary.
     '''
     assert_subclass_parse(
@@ -118,7 +119,8 @@ def test_intrinsic_function(f2003_create):
         expected_str='SIN(x)')
 
 
-def test_constant(f2003_create):
+@pytest.mark.usefixtures("f2003_create")
+def test_constant():
     '''Test that Constant types are matched by Primary.
     '''
     assert_subclass_parse(
@@ -127,7 +129,8 @@ def test_constant(f2003_create):
         expected_str='1.2E-03')
 
 
-def test_designator(f2003_create):
+@pytest.mark.usefixtures("f2003_create")
+def test_designator():
     '''Test that Designator types are matched by Primary.
     '''
     assert_subclass_parse(
@@ -136,7 +139,8 @@ def test_designator(f2003_create):
         expected_str='array(1 : 5)')
 
 
-def test_array_constructor(f2003_create):
+@pytest.mark.usefixtures("f2003_create")
+def test_array_constructor():
     '''Test that Array Constructor types are matched by Primary.
     '''
     assert_subclass_parse(
@@ -145,21 +149,43 @@ def test_array_constructor(f2003_create):
         expected_str='[1.2, 2.3E+2, - 5.1E-3]')
 
 
-def test_structure_constructor(f2003_create):
+@pytest.mark.usefixtures("f2003_create")
+def test_structure_constructor_1():
     '''Test that Structure Constructor types are matched by Primary.
+
     '''
-    # Note: The actual returned type is Data_Ref. With more context of what
-    # has already been parsed it is possible that this could change in the
-    # future. For instance, in this example "PERSON" could actually be the
-    # name of a function.
     assert_subclass_parse(
-        'PERSON ( 12,   "Jones" )', f2003.Structure_Constructor,
-        actual_type=f2003.Part_Ref,
+        "PERSON ( 12,   \"Jones\" )", f2003.Structure_Constructor,
+        actual_type=f2003.Structure_Constructor,
         expected_str='PERSON(12, "Jones")')
 
 
 @pytest.mark.xfail(reason="Requires more parse context (#190)")
-def test_function_reference(f2003_create):
+@pytest.mark.usefixtures("f2003_create")
+def test_structure_constructor_2():
+    '''Test that Structure Constructor types are matched by Primary.
+
+    This test currently fails as PERSON( 12, 24 ) is matched as an
+    array access (Designator). In general, the only way to tell the
+    difference is to know whether PERSON is the name of a structure or
+    an array. However, we don't keep this information at the moment
+    and even when we do in the future, we still might
+    not know the information before link time as declarations may be
+    in different modules (see issue #201).
+
+    '''
+    # This test incorrectly Matches with Designator. It would
+    # correctly match with Structure_Constructor but Designator is
+    # checked first from the Primary class.
+    assert_subclass_parse(
+        "PERSON ( 12, 24 )", f2003.Structure_Constructor,
+        actual_type=f2003.Structure_Constructor,
+        expected_str='PERSON(12, 24)')
+
+
+@pytest.mark.xfail(reason="Requires more parse context (#190)")
+@pytest.mark.usefixtures("f2003_create")
+def test_function_reference():
     '''This test demonstrates the inability to distinguish
     Structure_Constructor from Function_Reference without more parse context
     than is currently being provided.
@@ -187,39 +213,66 @@ def test_type_param_name():
         expected_str='INTEGER')
 
 
-def test_parenthesis(f2003_create):
-    '''Test that Parenthesis types are matched by Primary.
+@pytest.mark.parametrize("string", ["(a)", "(a + b)", "(a + 1)", "((a))",
+                                    "(\"a\" + \"c\")", "(\"a\" + \")\")",
+                                    "(')' + \")\")"])
+@pytest.mark.usefixtures("f2003_create")
+def test_parenthesis(string):
+    '''Test that Parenthesis types are matched by the Primary. As
+    fparser2 implements this match as a separate class called
+    `Parenthesis`, also check this class directly.
+
     '''
     assert_subclass_parse(
-        '(a +  b)', f2003.Parenthesis,
-        expected_str='(a + b)')
+        string, f2003.Parenthesis,
+        expected_str=string)
+
+    result = f2003.Parenthesis(string)
+    assert isinstance(result, f2003.Parenthesis)
+    assert str(result) == string
 
 
-def test_no_match(f2003_create):
+@pytest.mark.parametrize("string", ["(a+b)*(c+d)", "()"])
+@pytest.mark.parametrize("cls", [f2003.Primary, f2003.Parenthesis])
+@pytest.mark.usefixtures("f2003_create")
+def test_parenthesis_no_match(string, cls):
+    '''Test that invalid Parenthesis input is not matched by Primary or
+    Parenthesis classes.
+
+    '''
+    with pytest.raises(NoMatchError) as error:
+        _ = cls(string)
+    assert "{0}: '{1}'".format(cls.__name__, string) in str(error.value)
+
+
+@pytest.mark.usefixtures("f2003_create")
+def test_no_match():
     '''Test that a NoMatchError is raised if we provide code
     that isn't allowed as a Primary type (e.g. a comment).
     '''
-    with pytest.raises(fparser.two.utils.NoMatchError):
+    with pytest.raises(NoMatchError):
         _ = f2003.Primary('! A comment')
 
 
 @pytest.mark.xfail(reason="Requires more parse context (#190)")
-def test_c701_no_assumed_size_array(f2003_create):
+@pytest.mark.usefixtures("f2003_create")
+def test_c701_no_assumed_size_array():
     '''Test C701 (R701) The type-param-name shall be the name of a type.
     This test cannot be passed without more parse context of things like
     defined types.
     '''
     context = f2003.Type_Declaration_Stmt("INTEGER :: not_a_type")
-    with pytest.raises(fparser.two.utils.NoMatchError):
+    with pytest.raises(NoMatchError):
         f2003.Primary('not_a_type',)  # context)
 
 
 @pytest.mark.xfail(reason="Requires more parse context (#190)")
-def test_c702_no_assumed_size_array(f2003_create):
+@pytest.mark.usefixtures("f2003_create")
+def test_c702_no_assumed_size_array():
     '''Test C702 (R701) The designator shall not be a whole assumed-size array.
     This test cannot be passed without more parse context of things like
     defined types.
     '''
     context = f2003.Type_Declaration_Stmt("integer(*) :: assumed_size_array")
-    with pytest.raises(fparser.two.utils.NoMatchError):
+    with pytest.raises(NoMatchError):
         f2003.Primary('assumed_size_array',)  # context)
