@@ -1,4 +1,4 @@
-# Copyright (c) 2018 Science and Technology Facilities Council
+# Copyright (c) 2018-2021 Science and Technology Facilities Council
 
 # All rights reserved.
 
@@ -38,8 +38,10 @@ program statement.
 '''
 
 import pytest
+from fparser.api import get_reader
 from fparser.two.utils import NoMatchError
-from fparser.two.Fortran2003 import Program_Stmt
+from fparser.two.symbol_table import SYMBOL_TABLES
+from fparser.two.Fortran2003 import Program_Stmt, Program, Name
 
 
 @pytest.mark.usefixtures("f2003_create")
@@ -63,3 +65,28 @@ def test_invalid():
         with pytest.raises(NoMatchError) as excinfo:
             _ = Program_Stmt(string)
         assert "Program_Stmt: '{0}'".format(string) in str(excinfo.value)
+
+
+@pytest.mark.usefixtures("f2003_create")
+def test_prog_symbol_table():
+    ''' Check that an associated symbol table is created when parsing a
+    program unit. '''
+    reader = get_reader("program my_prog\n"
+                        "end program my_prog\n")
+    prog = Program(reader)
+    assert "my_prog" in SYMBOL_TABLES._symbol_tables
+
+
+def test_get_name():
+    """Test we can get the name of the program
+    """
+    obj = Program_Stmt("program foo")
+    assert obj.get_name() == Name("foo")
+
+
+def test_get_start_name():
+    """Test we can get the name of the function as a string
+    """
+
+    obj = Program_Stmt("program foo")
+    assert obj.get_start_name() == "foo"
