@@ -47,7 +47,7 @@ from fparser.scripts import fparser2, read
 # into runner() as an argument options class in subsequent tests
 
 
-class DummyArgs(object):
+class DummyArgs():
     ''' dummy object pretending to be the argument options '''
     mode = "auto"
     task = "show"
@@ -108,7 +108,7 @@ def test_runner_set_mode(tmpdir, capsys):
     # Create a dummy class with the required attribute to pass into
     # runner() as an argument options class
 
-    class DummyArgsFree(object):
+    class DummyArgsFree():
         ''' dummy object pretending to be the argument options '''
         mode = "free"
         task = "show"
@@ -167,7 +167,7 @@ def test_runner_output_task_show(tmpdir, capsys):
     'task' option set to "show".
 
     '''
-    class DummyArgsTask(object):
+    class DummyArgsTask():
         ''' dummy object pretending to be the argument options '''
         mode = "free"
         task = "show"
@@ -190,7 +190,7 @@ def test_runner_output_task_repr(tmpdir, capsys):
     has parsed with the 'task' option set to "repr".
 
     '''
-    class DummyArgsTask(object):
+    class DummyArgsTask():
         ''' dummy object pretending to be the argument options '''
         mode = "free"
         task = "repr"
@@ -215,7 +215,7 @@ def test_runner_output_task_none(tmpdir, capsys):
     to "none".
 
     '''
-    class DummyArgsTask(object):
+    class DummyArgsTask():
         ''' dummy object pretending to be the argument options '''
         mode = "free"
         task = "none"
@@ -275,10 +275,11 @@ def test_runner_multi_output_except(tmpdir, capsys):
 # fparser2.py script function main()
 
 
-def test_main_output_task_default(tmpdir, capsys, monkeypatch):
+def test_main_output_task_default(tmpdir, capsys, monkeypatch, log):
     '''Test that the script main() function outputs the code it has parsed
-    by default.'''
+    by default and that no errors are logged.'''
     import sys
+    log.reset()
     # Create a temporary file containing Fortran code to pass into runner()
     my_file = tmpdir.mkdir("sub").join("hello.f90")
     my_file.write("program hello\nend program hello\n")
@@ -290,6 +291,9 @@ def test_main_output_task_default(tmpdir, capsys, monkeypatch):
     stdout, stderr = capsys.readouterr()
     assert stdout == "PROGRAM hello\nEND PROGRAM hello\n"
     assert "File: '" in stderr and "hello.f90'" in stderr
+    # Check that nothing has been logged (apart from some 'debug' msgs)
+    for log_level in ["info", "warning", "error", "critical"]:
+        assert log.messages[log_level] == []
 
 
 def test_main_output_task_show(tmpdir, capsys, monkeypatch):
@@ -452,7 +456,7 @@ def test_main_output_std_invalid(capsys, monkeypatch):
 # into runner() as an argument options class in subsequent tests
 
 
-class DummyReadArgs(object):
+class DummyReadArgs():
     '''dummy object pretending to be the argument options for the read
     module in scripts.
 
